from collections import OrderedDict  # TODO: pryč s tím
import itertools

from shlex import shlex
import symbols


# TODO: slovník sestavit až sekundárně (takže nedědit)
# TODO: podporovat append, del a přístup pod jménem/klíčem
class XmlTag(OrderedDict):
    def __init__(self, lines=None, tag=None, opening=None, empty=None):
        super().__init__()
        self.lines = lines
        self.tag = tag
        self.opening = opening
        self.empty = empty

    @property
    def closing(self):
        return not self.opening

    def add(self, attr, value):
        existing = self.get(attr, None)
        if existing is not None:
            if isinstance(existing, str):
                self[attr] = [existing, value]
            else:
                existing.append(value)
        else:
            self[attr] = value

    def __str__(self):
        opening = '' if self.opening else '/'
        empty = '/' if self.empty else ''
        # ztratil jsem původní pořadí duplikovaných atributů
        attrs = ''.join(
            ' {}="{}"'.format(attr, val) for attr, val in self.items())
        return '<{opening}{tag}{attrs}{empty}>'.format(
            opening=opening, tag=self.tag, attrs=attrs, empty=empty)

    def __repr__(self):
        return 'XmlTag(' + symbols.escape(self) + ')'

    @classmethod
    def parse(cls, lines):
        parser = shlex(lines[1:])
        xml = cls()
        attr = None
        value = None
        while True:
            token = parser.get_token()  # ValueError("No closing quotation")
            if not token:
                break
            elif xml.opening is None:
                xml.opening = token != '/'
                if xml.opening:
                    xml.tag = token
            elif xml.tag is None:
                if token == '>':
                    break
                xml.tag = token
            elif attr is None:
                if token == '/':
                    xml.empty = True
                elif token == '>':
                    break
                elif token == '=':
                    break  # unexpected =
                else:
                    attr = token
                    value = '='  # expecting =
            elif xml.empty:
                if token == '>':
                    break
                else:
                    break  # unexpected /
            elif value == '=':
                if token == '=':
                    value = None  # expecting a value
                elif token == '/':
                    xml.add(attr, None)
                    xml.empty = True
                elif token == '>':
                    xml.add(attr, None)
                    break
                else:
                    xml.add(attr, None)  # got an empty value
                    attr = token  # immediately followed by another
            elif value is None:
                if (len(token) > 1 and token[0] in '\'"' and
                        token[0] == token[-1]):
                    token = token[1:-1]
                xml.add(attr, token)
                attr = None  # expecting an attribute
                value = None
        return xml


def extract_attributes_from_tag(tag):
    if ',' in tag:
        tags = tag.split(',')
        if '' not in tags:
            return merge_ambiguous_tags(tags)
    if len(tag) % 2 != 0:
        raise ValueError('Malformed tag "{}"'.format(tag))
    return grouper(tag, 2)


def merge_ambiguous_tags(tags):
    merged = {}
    for attr, val in itertools.chain.from_iterable(
            extract_attributes_from_tag(tag) for tag in tags):
        if attr not in merged:
            merged[attr] = val
        elif isinstance(merged[attr], str):
            if val != merged[attr]:
                merged[attr] = [merged[attr], val]
        elif val not in merged[attr]:
            merged[attr].append(val)
    return merged


# copied from oVirt/VDSM, which in turn used
# http://docs.python.org/2.6/library/itertools.html?highlight=grouper#recipes
def grouper(iterable, n, fillvalue=None):
    "Collect data into fixed-length chunks or blocks"
    # grouper('ABCDEFG', 3, 'x') --> ABC DEF Gxx
    args = [iter(iterable)] * n
    return itertools.zip_longest(fillvalue=fillvalue, *args)
